package com.zsc.edu.gateway.framework.message.websocket;

import org.springframework.stereotype.Component;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Supplier;

/**
 * @author zhuang
 */
@Component
public class GenericWebSocketHandler extends TextWebSocketHandler {

    private final ExecutorService executorService = Executors.newCachedThreadPool();

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        // 从 session 中获取业务逻辑的 Supplier
        Supplier<Object> dataSupplier = (Supplier<Object>) session.getAttributes().get("dataSupplier");

        if (dataSupplier == null) {
            session.close();
            return;
        }

        AtomicBoolean isCompleted = new AtomicBoolean(false);

        executorService.execute(() -> {
            try {
                while (!isCompleted.get()) {
                    Object data = dataSupplier.get();
                    if (data == null) {
                        break;
                    }
                    if (!isCompleted.get()) {
                        session.sendMessage(new TextMessage(data.toString()));
                    }
                    Thread.sleep(5000); // 每隔 5 秒发送一次数据
                }
            } catch (IOException | InterruptedException e) {
                try {
                    session.close();
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            } finally {
                try {
                    session.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
    }
}