package com.zsc.edu.gateway.framework.message.websocket;

import com.zsc.edu.gateway.modules.iot.device.service.DeviceService;
import com.zsc.edu.gateway.modules.iot.product.service.ProductService;
import com.zsc.edu.gateway.modules.iot.record.service.RecordDataService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.server.HandshakeInterceptor;

import java.util.Map;
import java.util.function.Supplier;

/**
 * @author lenovo
 */
public class WebSocketInterceptor implements HandshakeInterceptor {
    @Resource
    private DeviceService deviceService;
    @Resource
    private RecordDataService recordService;
    @Resource
    private ProductService productService;

    @Override
    public boolean beforeHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Map<String, Object> attributes) {
        if (request instanceof ServletServerHttpRequest) {
            HttpServletRequest servletRequest = ((ServletServerHttpRequest) request).getServletRequest();
            // 获取完整路径
            String path = servletRequest.getRequestURI();

            // 根据路径设置不同的业务逻辑 Supplier
            switch (path) {
                case "/api/rest/device/ws/device/status":
                    attributes.put("dataSupplier", (Supplier<String>) () -> String.valueOf(deviceService.status()));
                    break;
                case "/api/rest/device/ws/record/status":
                    attributes.put("dataSupplier", (Supplier<String>) () -> String.valueOf(recordService.recordDataStatus()));
                    break;
                case "/api/rest/product/ws/product/status":
                    attributes.put("dataSupplier", (Supplier<String>) () -> String.valueOf(productService.status()));
                    break;
                default:
                    attributes.put("dataSupplier", (Supplier<String>) () -> "Unknown path: " + path);
                    break;
            }
        }
        return true;
    }

    @Override
    public void afterHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Exception exception) {
        // 握手完成后不需要额外操作
    }
}